(* Write TPTP output files from Metis problems. *)
(* Currently, equality is not handled, so it has to be manually replaced. *)

module Metis_tptp = struct

let rec oiter oc fn sep = function
    [] -> ()
  | [e] -> fn oc e
  | h :: t -> fn oc h; output_string oc sep; oiter oc fn sep t;;

let rec write_tm oc = function
    Meson.Fvar n -> Printf.fprintf oc "V%i" n
  | Meson.Fnapp (i,[]) -> Printf.fprintf oc "c%i" i
  | Meson.Fnapp (i,l) ->
      Printf.fprintf oc "c%i(" i;
      oiter oc write_tm "," l;
      Printf.fprintf oc ")";;

let rec fol_frees sf = function
    Meson.Fvar x -> insert x sf
  | Meson.Fnapp(_,l) ->
      List.fold_left fol_frees sf l;;
      let fol_frees_lits = List.fold_left
        (fun sf (_, t) -> List.fold_left fol_frees sf t) [];;

let write_lit oc (i, l) =
  (if i < 0 then Printf.fprintf oc "~");
  Printf.fprintf oc "p%i"  (abs i);
  if l <> [] then (Printf.fprintf oc "("; oiter oc write_tm "," l; Printf.fprintf oc ")");;

let write_lits oc l = Printf.fprintf oc "("; oiter oc write_lit " | " l; Printf.fprintf oc ")";;

let write_uni_lits oc l =
  let vs = fol_frees_lits l in
  if vs <> [] then (Printf.fprintf oc "!["; oiter oc (fun _ i -> Printf.fprintf oc "V%i" i) "," vs; Printf.fprintf oc "]: ");
  write_lits oc l;
;;

let clno = ref 0;;
let write_clause oc fl =
  incr clno;
  Printf.fprintf oc "fof(a%i, axiom, " !clno;
  write_uni_lits oc fl;
  Printf.fprintf oc ").\n"
;;

let fol_of_hol_clause th =
  let lconsts = freesl (hyp th) in
  let tm = concl th in
  let hlits = disjuncts tm in
  let flits = map (Meson.fol_of_literal [] lconsts) hlits in
  flits
;;

let cnt = ref 0;;
let write_problem fll =
  let fll = List.map fol_of_hol_clause fll in
  let concls, axs = List.partition (fun cl -> List.mem (-2, []) cl) fll in
  let axs = List.filter (fun x -> x <> [2, []]) axs in
  let concls = List.map (List.filter (fun x -> x <> (-2, []))) concls in
  incr cnt;
  let oc = open_out ("/tmp/holp" ^ (string_of_int !cnt) ^ ".p") in
  List.iter (write_clause oc) axs;
  Printf.fprintf oc "fof(concl, conjecture, ~(";
  oiter oc write_uni_lits " & " concls;
  Printf.fprintf oc ")).\n";
  close_out oc
;;

end
