(* ========================================================================= *)
(* CNF PROBLEMS                                                              *)
(* Copyright (c) 2001 Joe Leslie-Hurd, distributed under the MIT license     *)
(* ========================================================================= *)

open Useful;;

(* ------------------------------------------------------------------------- *)
(* Problems.                                                                 *)
(* ------------------------------------------------------------------------- *)

type problem =
     {axioms : Thm.clause list;
      conjecture : Thm.clause list};;

let toClauses {axioms=axioms;conjecture=conjecture} = axioms @ conjecture;;

(*let size prob =
      let lits (cl,n) = n + Literal.Set.size cl

      in let syms (cl,n) = n + Literal.Set.symbols cl

      in let typedSyms (cl,n) = n + Literal.Set.typedSymbols cl

      in let cls = toClauses prob
    in
      {clauses = length cls;
       literals = Mlist.foldl lits 0 cls;
       symbols = Mlist.foldl syms 0 cls;
       typedSymbols = Mlist.foldl typedSyms 0 cls}
    ;;*)

let freeVars {axioms=axioms;conjecture=conjecture} =
    Name.Set.union
      (Literal.Set.freeVarsList axioms)
      (Literal.Set.freeVarsList conjecture);;

  let clauseToFormula cl =
      Formula.listMkDisj (Literal.Set.transform Literal.toFormula cl);;
  let toFormula prob =
      Formula.listMkConj (List.map clauseToFormula (toClauses prob));;

  let toGoal {axioms=axioms;conjecture=conjecture} =
        let clToFm x = Formula.generalize (clauseToFormula x)
        in let clsToFm x = Formula.listMkConj (List.map clToFm x)

        in let fm = Formula.False
        in let fm =
            if Mlist.null conjecture then fm
            else Formula.Imp (clsToFm conjecture, fm)
        in let fm = Formula.Imp (clsToFm axioms, fm)
      in
        fm
      ;;


(* ------------------------------------------------------------------------- *)
(* Categorizing problems.                                                    *)
(* ------------------------------------------------------------------------- *)

type propositional =
    Propositional
  | Effectively_propositional
  | Non_propositional;;

type equality =
    Non_equality
  | Equality
  | Pure_equality;;

type horn =
    Trivial
  | Unit
  | Double_horn
  | Horn
  | Negative_horn
  | Non_horn;;

type category =
     {propositional : propositional;
      equality : equality;
      horn : horn};;

let categorize prob =
      let cls = toClauses prob

      in let rels =
            let f (cl,set) = Name_arity.Set.union set (Literal.Set.relations cl)
          in
            Mlist.foldl f Name_arity.Set.empty cls

      in let funs =
            let f (cl,set) = Name_arity.Set.union set (Literal.Set.functions cl)
          in
            Mlist.foldl f Name_arity.Set.empty cls

      in let propositional =
          if Name_arity.Set.allNullary rels then Propositional
          else if Name_arity.Set.allNullary funs then Effectively_propositional
          else Non_propositional

      in let equality =
          if not (Name_arity.Set.member Atom.eqRelation rels) then Non_equality
          else if Name_arity.Set.size rels = 1 then Pure_equality
          else Equality

      in let horn =
          if List.exists Literal.Set.null cls then Trivial
          else if Mlist.all (fun cl -> Literal.Set.size cl = 1) cls then Unit
          else
              let pos cl = Literal.Set.count Literal.positive cl <= 1
              in let neg cl = Literal.Set.count Literal.negative cl <= 1
            in
              match (Mlist.all pos cls, Mlist.all neg cls) with
                (true,true) -> Double_horn
              | (true,false) -> Horn
              | (false,true) -> Negative_horn
              | (false,false) -> Non_horn
    in
      {propositional = propositional;
       equality = equality;
       horn = horn}
    ;;

let categoryToString {propositional=propositional;equality=equality;horn=horn} =
    (match propositional with
       Propositional -> "propositional"
     | Effectively_propositional -> "effectively propositional"
     | Non_propositional -> "non-propositional") ^
    ", " ^
    (match equality with
       Non_equality -> "non-equality"
     | Equality -> "equality"
     | Pure_equality -> "pure equality") ^
    ", " ^
    (match horn with
       Trivial -> "trivial"
     | Unit -> "unit"
     | Double_horn -> "horn (and negative horn)"
     | Horn -> "horn"
     | Negative_horn -> "negative horn"
     | Non_horn -> "non-horn");;

